from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import Select
from wasabi import msg
import concurrent.futures
import requests
import time
import re

# 票务链接
confetti_url = 'https://confetti-web.com/@/may-holic17'

# 测试用票务链接
# confetti_url = 'http://confetti-web.com/@/otonapro-58'

# 获取实际购票系统URL
r = requests.get(confetti_url)
html = r.text
pattern = r'https://www\.s2\.e-get\.jp/lrpnot/pt/&lg=-1&s=[\w-]+'
ticket_url = re.findall(pattern, html)[0]
print(ticket_url)


def ticket_tool(round_idx, cast_name, ticket_num):
    while True:
        # 构成log输出信息
        msg_head = f'[{round_idx}] {cast_name}×{ticket_num}'
        msg.info(f'{msg_head} - 开始抢票')
        # 初始化浏览器
        options = webdriver.ChromeOptions()
        # 禁用插件
        options.add_argument('--disable-extensions')
        # 默认执行完不关闭窗口
        options.add_experimental_option('detach', True)
        driver = webdriver.Chrome(options=options)
        # 进入购票界面
        driver.get(ticket_url)
        # 处理服务器忙
        while True:
            if '/pt/' not in driver.current_url:
                msg.warn(f'{msg_head} - 服务器忙，准备重试')
                time.sleep(1)
                driver.get(ticket_url)
            else:
                break
        # 开始抢票
        try:
            # 进入场次选择页面
            driver.find_element(By.CSS_SELECTOR, 'a.btn_reserve').click()
            # 选择场次
            rounds = driver.find_elements(By.CSS_SELECTOR, 'table#seat_condition tbody tr')
            target_row = rounds[round_idx + 1]
            # 进入特典选择页面
            target_row.find_element(By.CSS_SELECTOR, 'td a.conditon').click()
            # 选择特典cast
            casts = driver.find_elements(By.CSS_SELECTOR, 'table.reserve_number tbody tr')
            for cast in casts:
                if cast_name in cast.text:
                    select = Select(cast.find_element(By.CSS_SELECTOR, 'select.number'))
                    select.select_by_value(str(ticket_num))
                    break
            # 提交，进入支付流程
            driver.find_element(By.CSS_SELECTOR, 'a.footer_next').click()
            # 判断提交结果
            if '/pt/cart.act' in driver.current_url:
                msg.good(f'{msg_head} - 抢票成功，待支付')
                # 修改窗口标题，便于管理
                driver.execute_script(f"document.title = '{msg_head}'")
                break
            else:
                msg.fail(f'{msg_head} - 抢票失败，准备重试')
                driver.quit()
        # 错误处理
        except Exception as e:
            msg.fail(f'{msg_head} - 发生错误：{e}')
            driver.quit()


# 抢票配置
config = [
    [1, '小日向', 2]
]

# 多线程
with concurrent.futures.ThreadPoolExecutor(max_workers=len(config)) as executor:
    futures = [executor.submit(ticket_tool, *args) for args in config]
    concurrent.futures.wait(futures)
